;												
; Seismograph									
; Beschleunigungssensor BMA020					
; DOG-LCD 128x64 fr ATmega8 (single suply 3,3V)
; Bernhard.Erfurt@gmx.de						
; 09/2014										
;												
; 10x pro Sekunde den Beschleunigungssensor 	
; auslesen und Werte Anzeigen					
;												
; Anschlussbelegung PINS						
;												
; PB0 ==> LCD_RES								
; PB1 ==> LCD_A0								
; PB2 ==> LCD_CS								
; PB3 ==> LCD_SI								
; PB4 ==> 										
; PB5 ==> LCD_SCL								
; PB6 ==> Quarz									
; PB7 ==> Quarz									
;												
; PC0 ==> 										
; PC1 ==> LED GELB								
; PC2 ==> 										
; PC3 ==> 										
; PC4 ==> I2C-SDA								
; PC5 ==> I2C-SCL								
;												
; PD0 ==> TASTE-A								
; PD1 ==> USART-TX								
; PD2 ==> TASTE-B								
; PD3 ==> TASTE-C								
; PD4 ==> TASTE-D								
; PD5 ==> LED GRN								
; PD6 ==> LED ROT								
; PD7 ==> LCD-Hintergrundbeleuchtung			
;												


; ATmega8 										
.include "m8def.inc"

; KONSTANTEN									
.equ TAKT		=16000000			; 16 MHz	
; USART											
.equ BAUD		= 9600
; TIMER											
.equ TIMERWERT1	=65536-6250		;(2 hoch 16 minus....15625) bei 16MHz + Vorteiler 1024 => 1s 	
; I2C / TWI MASTER EINSTELLUNGEN				
.equ TWI_BIT_RATE			= 100		; 255	; Bit-Rate							
.equ TWI_PRESCALER			= 1			; 1		; Vorteiler: 0=1 1=4 2=16 3=64		
; SLAVE BMA020									
.equ ADRESSE_BMA020			= 56
.equ ERROR_BMA020_MESSUNG	= 66
; TWI-FEHLER-CODES								
.equ ERROR_TWI_MT_START		= 11		; ERROR-CODE	
.equ ERROR_TWI_MT_DATA		= 12		; ERROR-CODE	
.equ ERROR_TWI_MR_START		= 21		; ERROR-CODE	
.equ ERROR_TWI_MR_DATA		= 22		; ERROR-CODE	
.equ ERROR_TWI_MR_DATA_LAST	= 23		; ERROR-CODE	
; TASTE "MENUE-PLUS"							
.equ TASTE_MENUE_PLUS_PIN_NR		= 0
.equ TASTE_MENUE_PLUS_PIN			= PIND
.equ TASTE_MENUE_PLUS_DDR			= DDRD
.equ TASTE_MENUE_PLUS_PORT			= PORTD
; TASTE "MENUE-MINUS"							
.equ TASTE_MENUE_MINUS_PIN_NR		= 2
.equ TASTE_MENUE_MINUS_PIN			= PIND
.equ TASTE_MENUE_MINUS_DDR			= DDRD
.equ TASTE_MENUE_MINUS_PORT			= PORTD
; TASTE "PLUS"									
.equ TASTE_PLUS_PIN_NR				= 3
.equ TASTE_PLUS_PIN					= PIND
.equ TASTE_PLUS_DDR					= DDRD
.equ TASTE_PLUS_PORT				= PORTD
; TASTE "MINUS"									
.equ TASTE_MINUS_PIN_NR				= 4
.equ TASTE_MINUS_PIN				= PIND
.equ TASTE_MINUS_DDR				= DDRD
.equ TASTE_MINUS_PORT				= PORTD
; TASTEN-WERTE									
.equ TASTE_MENUE_PLUS	= 1
.equ TASTE_MENUE_MINUS	= 2
.equ TASTE_PLUS			= 4
.equ TASTE_MINUS		= 8
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRD
.equ LED_GRUEN_PORT					= PORTD
; LED-GELB										
.equ LED_GELB_PIN_NR				= 1
.equ LED_GELB_DDR					= DDRC
.equ LED_GELB_PORT					= PORTC
; LED-ROT										
.equ LED_ROT_PIN_NR					= 6
.equ LED_ROT_DDR					= DDRD
.equ LED_ROT_PORT					= PORTD
; LED-BELEUCHTUNG								
.equ LED_BEL_PIN_NR					= 7
.equ LED_BEL_DDR					= DDRD
.equ LED_BEL_PORT					= PORTD
; DOG-LCD (PORTB)								
.equ LCD_RES			= 0		; SLAVE-RESET	
.equ LCD_A0				= 1		; SLAVE-A0		
.equ LCD_CS				= 2		; MASTER CS		
.equ LCD_SI				= 3		; MASTER MOSI	
.equ LCD_SCL			= 5		; MASTER SCK	
; SRAM-BELEGUNG ATmega8							

.equ adr_JAHR					= SRAM_START+0
.equ adr_MONAT					= SRAM_START+1
.equ adr_TAG					= SRAM_START+2
.equ adr_hh						= SRAM_START+3
.equ adr_mm						= SRAM_START+4
.equ adr_ss						= SRAM_START+5
.equ adr_ss10					= SRAM_START+6
.equ adr_ALARM_SCHWELLE			= SRAM_START+7		; 8			
.equ adr_LCD_KONTRAST			= SRAM_START+8		; KONTRAST	

.equ adr_TASTE					= SRAM_START+10		; TASTE		
.equ adr_ALARM_WERT				= SRAM_START+11		; 0=AUS		

.equ adr_MENUE					= SRAM_START+12		; MENUE		
.equ adr_ERROR					= SRAM_START+13 
.equ adr_TWI_ERROR_SLAVES		= SRAM_START+14
.equ adr_BMA020_VORHANDEN		= SRAM_START+15 

.equ adr_FREIGABE_ss10			= SRAM_START+16
.equ adr_FREIGABE_ss			= SRAM_START+17
.equ adr_FREIGABE_mm			= SRAM_START+18
.equ adr_FREIGABE_hh			= SRAM_START+19
.equ adr_FREIGABE_TAG			= SRAM_START+20

.equ adr_BMA020_X_MSB			= SRAM_START+30
.equ adr_BMA020_X_LSB			= SRAM_START+31
.equ adr_BMA020_Y_MSB			= SRAM_START+32
.equ adr_BMA020_Y_LSB			= SRAM_START+33
.equ adr_BMA020_Z_MSB			= SRAM_START+34
.equ adr_BMA020_Z_LSB			= SRAM_START+35
.equ adr_BMA020_G_MSB			= SRAM_START+36
.equ adr_BMA020_G_LSB			= SRAM_START+37

.equ adr_BMA020_X_OLD_MSB		= SRAM_START+40
.equ adr_BMA020_X_OLD_LSB		= SRAM_START+41
.equ adr_BMA020_Y_OLD_MSB		= SRAM_START+42
.equ adr_BMA020_Y_OLD_LSB		= SRAM_START+43
.equ adr_BMA020_Z_OLD_MSB		= SRAM_START+44
.equ adr_BMA020_Z_OLD_LSB		= SRAM_START+45

.equ adr_BMA020_X_OFFS_MSB		= SRAM_START+46
.equ adr_BMA020_X_OFFS_LSB		= SRAM_START+47
.equ adr_BMA020_Y_OFFS_MSB		= SRAM_START+48
.equ adr_BMA020_Y_OFFS_LSB		= SRAM_START+49
.equ adr_BMA020_Z_OFFS_MSB		= SRAM_START+50
.equ adr_BMA020_Z_OFFS_LSB		= SRAM_START+51

.equ adr_BMA020_X				= SRAM_START+52
.equ adr_BMA020_Y				= SRAM_START+53
.equ adr_BMA020_Z				= SRAM_START+55

.equ adr_LCD_PAGE0				= SRAM_START+79		; oberste ZEILE (ganz links) (21 BYTES)							

.equ adr_LCD_DATENFELD			= SRAM_START+100	; LCD-Datenfeld (128 SPALTEN) / PAGE 1...7 =7x128 = 896 BYTES	

.equ adr_SRAM_CHECK				= SRAM_START+997	; SRAM CHECK muss immer "S" sein	

; EEPROM-BELEGUNG ATmega8						
.equ adr_EEPROM_DATEN			= 0


; REGISTER										
.def TWI_FEHLER_CODE	= R12
.def NULL 	= R13
.def EINS 	= R14
.def VOLL	= R15

.def temp 	= R16
.def temp1 	= R17
.def temp2 	= R18
.def temp3 	= R19
.def temp4 	= R20
.def temp5 	= R21
.def temp6 	= R22
.def temp7 	= R23
.def temp8 	= R24
.def temp9 	= R25


.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT_ERROR	;rjmp EXT_INT0   ; IRQ0 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       						
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF  			; Timer0 Overflow Handler   
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	


; Bei unkorrekten Interrupt aufrufen erfolgt ein RESET ber WDR	
INTERRUPT_ERROR:
	sbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN aus	
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus	
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	wdr	
INTERRUPT_ERROR_SCHLEIFE:
	rjmp INTERRUPT_ERROR_SCHLEIFE

RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM_CLEAR										
	rcall SRAM_CLEAR
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; VOLL				
	ldi temp,255
	mov VOLL,temp
	; SRAM CHECK		
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
	; KONTRAST			
	ldi temp,24
	STS(adr_LCD_KONTRAST),temp
	; ZEIT				
	ldi temp,23
	STS(adr_hh),temp
	ldi temp,59
	STS(adr_mm),temp
	ldi temp,30
	STS(adr_ss),temp
	; DATUM				
	ldi temp,31
	STS(adr_TAG)  ,temp
	ldi temp,12
	STS(adr_MONAT),temp
	ldi temp,14
	STS(adr_JAHR) ,temp
	; ALARM-Schwellwert
	ldi temp,8
	STS(adr_ALARM_SCHWELLE),temp
; PORT B+C+D alles EINGANG							
	out DDRB, NULL
	out PORTB,NULL
	out DDRC, NULL
	out PORTC,NULL
	out DDRD, NULL
	out PORTD,NULL
; LED_INITIALISIERUNG								
	rcall LED_INITIALISIERUNG
; WATCHDOG_AKTIVIEREN								
	rcall WATCHDOG_AKTIVIEREN
; NEUSTART_MIT_WDR									
	rcall NEUSTART_MIT_WDR
; LED_TEST											
	rcall LED_TEST
; TIMER1_INITIALISIERUNG							
	rcall TIMER1_INITIALISIERUNG
; TIMER aktivieren									
	rcall TIMER_AKTIVIEREN
; SPI_INITIALISIERUNG								
	rcall SPI_INITIALISIERUNG	
; LCD_INITIALISIERUNG								
	rcall LCD_INITIALISIERUNG
; TWI_INI											
	rcall TWI_INI
; USART_INITIALISIEREN								
	rcall USART_INITIALISIEREN
; TASTATUR_INITIALISIERUNG							
	rcall TASTATUR_INITIALISIERUNG
; Interrupts freigeben								
	sei
; ##############################################################################
; ##############################################################################
; ##############################################################################
	rcall EEPROM_KONFIGURATION_LADEN
	rcall EEPROM_KONFIGURATION_SPEICHERN
	rcall LCD_TEST
	cbi (LED_BEL_PORT),(LED_BEL_PIN_NR)			; Beleuchtung an	
	rcall LCD_BEGRUSSUNG
	rcall wait_1s
	rcall TWI_BUS_CHECK
	rcall wait_1s
	rcall BMA020_CHECK
	rcall wait_1s
	rcall BMA020_MESSUNG
	rcall BMA020_MESSUNG
	rcall BMA020_MESSUNG
	rcall BMA020_OFFSET_SCHNELL
	sbi (LED_BEL_PORT),(LED_BEL_PIN_NR)			; Beleuchtung aus
	; SEKUNDE			
	STS(adr_ss),NULL
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR						; Watch-Dog-RESET	
	LDS temp,(adr_FREIGABE_ss10)
	tst temp
	breq LOOP
;-------------------------------------------------------------------------------
LOOP_START:
	rcall BMA020_MESSUNG
	rcall BMA020_OFFSET_AUTO
	rcall BMA020_ALARM
	rcall MENUE

	rcall DIAGRAMM_ZEICHNEN_XYZ
	rcall LCD_ZEICHNEN
	rcall USART_OUTPUT_DATA
	rcall PROGRAMM_CHECK
	rcall TASTEN_WAIT
	rcall EEPROM_KONFIGURATION_SPEICHERN_AUTO
	; FREIGABE NULL	
	STS(adr_FREIGABE_ss10),NULL
	STS(adr_FREIGABE_ss),NULL
	STS(adr_FREIGABE_mm),NULL
	STS(adr_FREIGABE_hh),NULL
	STS(adr_FREIGABE_TAG),NULL
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "SENSOR_SPI.asm"
.include "SENSOR_LCD_ini.asm"
.include "SENSOR_LCD_MENUE.asm"
.include "SENSOR_TIMER.asm"
.include "SENSOR_EEPROM.asm"
.include "SENSOR_SONSTIGES.asm"
.include "SENSOR_I2C_MASTER.asm"	; I2C-MASTER								
.include "SENSOR_I2C_BMA020.asm"	; I2C-BMA020	Beschleunigung	
.include "SENSOR_FUNCTION.asm"
.include "SENSOR_USART.asm"
.include "SENSOR_WAIT_16MHZ.asm"
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEXT_BEGRUESSUNG:
.db "BMA020 09/2014 V1",0
;-------------------------------------------------------------------------------
TABELLE_ASCII:
.db 0,0,0,0,0,0	; 0
.db 128,128,128,128,128,0	; 1
.db 128,128,128,128,128,0	; 2
.db 128,128,128,128,128,0	; 3
.db 128,128,128,128,128,0	; 4
.db 128,128,128,128,128,0	; 5
.db 128,128,128,128,128,0	; 6
.db 128,128,128,128,128,0	; 7
.db 128,128,128,128,128,0	; 8
.db 128,128,128,128,128,0	; 9

.db 128,128,128,128,128,0	; 10
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 

.db 128,128,128,128,128,0	; 20
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 
.db 128,128,128,128,128,0	; 

.db 128,128,128,128,128,0	; 30
.db 128,128,128,128,128,0	; 31
.db 0,0,0,0,0,0 			; (32)
.db 0,0,95,0,0,0 			;!(33)
.db 0,3,0,3,0,0 			;"(34)
.db 20,127,20,127,20,0 		;#(35)
.db 36,74,127,82,36,0 		;$(36)
.db 34,21,42,84,34,0 		;%(37)
.db 32,92,90,36,80,0 		;&(38)
.db 0,0,3,0,0,0 			;'(39)

.db 0,28,34,65,0,0 			;((40)
.db 0,65,34,28,0,0 			;)(41)
.db 34,20,62,20,34,0 		;*(42)
.db 8,8,62,8,8,0 			;+(43)
.db 0,64,32,0,0,0 			;,(44)
.db 0,8,8,8,0,0 			;-(45)
.db 0,64,0,0,0,0 			;.(46)
.db 32,16,8,4,2,0 			;/(47)
.db 62,65,65,65,62,0 		;0(48)
.db 4,2,127,0,0,0 			;1(49)

.db 66,97,81,73,70,0 		;2(50)
.db 34,65,73,73,54,0 		;3(51)
.db 8,12,10,127,8,0 		;4(52)
.db 47,69,69,69,57,0 		;5(53)
.db 60,74,73,73,48,0 		;6(54)
.db 1,97,25,5,3,0 			;7(55)
.db 54,73,73,73,54,0 		;8(56)
.db 6,9,73,41,30,0	 		;9(57)
.db 0,0,36,0,0,0 			;:(58)
.db 0,100,0,0,0,0 			;;(59)

.db 8,20,20,34,34,0 		;<(60)
.db 20,20,20,20,20,0 		;=(61)
.db 34,34,20,20,8,0 		;>(62)
.db 2,1,81,9,6,0 			;?(63)
.db 62,65,73,85,94,0 		;@(64)
.db 126,9,9,9,126,0 		;A(65)
.db 127,73,73,73,54,0 		;B(66)
.db 62,65,65,65,34,0 		;C(67)
.db 127,65,65,65,62,0  		;D(68)
.db 127,73,73,65,65,0  		;E(69)
.db 127,9,9,9,9,0  			;F(70)

.db 62,65,81,81,50,0 		;G(71)
.db 127,8,8,8,127,0  		;H(72)
.db 0,0,127,0,0,0  			;I(73)
.db 48,64,64,64,63,0  		;J(74)
.db 127,8,20,34,65,0  		;K(75)
.db 127,64,64,64,64,0  		;L(76)
.db 127,6,24,6,127,0  		;M(77)
.db 127,6,8,48,127,0  		;N(78)
.db 28,34,65,34,28,0  		;O(79)

.db 127,9,9,9,6,0 ;P(80)
.db 28,34,81,34,92,0 ;Q(81)
.db 127,9,25,41,70,0 ;R(82)
.db 38,73,73,73,50,0 ;S(83)
.db 1,1,127,1,1,0 ;T(84)
.db 63,64,64,64,63,0 ;U(85)
.db 3,28,96,28,3,0 ;V(86)
.db 63,64,60,64,63,0 ;W(87)
.db 65,54,8,54,65,0 ;X(88)
.db 1,6,120,6,1,0 ;Y(89)

.db 97,81,73,69,67,0 ;Z(90)
.db 127,65,65,0,0,0 ;[(91)
.db 0,3,28,96,0,0 ;\(92)
.db 0,0,65,65,127,0 ;](93)
.db 4,2,1,2,4,0 ;^(94)
.db 64,64,64,64,64,0 ;_(95)
.db 0,0,1,2,0,0 ;`(96)
.db 52,74,74,42,124,0 ;a(97)
.db 127,80,72,72,48,0 ;b(98)
.db 60,66,66,66,36,0 ;c(99)

.db 48,72,72,80,127,0 ;d(100)
.db 60,74,74,74,44,0 ;e(101)
.db 8,126,9,1,0,0 ;f(102)
.db 36,74,74,68,62,0 ;g(103)
.db 127,8,4,4,120,0 ;h(104)
.db 0,0,122,0,0,0 ;i(105)
.db 0,64,61,0,0,0 ;j(106)
.db 126,16,24,100,0,0 ;k(107)
.db 127,0,0,0,0,0 ;l(108)
.db 124,4,120,4,120,0 ;m(109)

.db 124,8,4,4,120,0 ;n(110)
.db 56,68,68,68,56,0 ;o(111)
.db 126,18,18,12,0,0 ;p(112)
.db 0,12,18,18,126,0 ;q(113)
.db 126,4,2,2,4,0 ;r(114)
.db 72,84,84,84,36,0 ;s(115)
.db 4,4,126,4,4,0 ;t(116)
.db 62,64,64,64,126,0 ;u(117)
.db 2,28,96,28,2,0 ;v(118)
.db 60,64,48,64,60,0 ;w(119)

.db 68,40,16,40,68,0 ;x(120)
.db 6,8,112,8,6,0 ;y(121)
.db 68,100,84,76,68,0 ;z(122)
.db 8,54,65,0,0,0 ;{(123)
.db 0,0,126,0,0,0 ;|(124)
.db 0,0,65,54,8,0 ;}(125)
.db 16,8,16,16,8,0 ;}(125)


